package ballwar.controller;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import util.*;
import ballwar.view.*;
import ballwar.model.*;
import ballwar.model.strategy.*;
import ballwar.model.paint.*;


/**
 * The "Controller" in a Model-View-Controller architecture
 * Sets up the appropriate wiring between the model and the view.
 */
public class BallControl extends JApplet {
  
  /**
   * The system's view (GUI).
   */
  private BallGUI _view;
  /**
   * The window listener used by the view when the window is closed.
   * By default, it does nothing, as needed for an applet.   One of the 
   * constructors can be used to set this value to 
   * something else, as needed when run as an application.
   */
  private WindowListener _windowListener = new WindowAdapter() {};
  
  /**
   * The system's model.
   */
  private BallModel _model;
  
  /**
   * An OptionPanel used to display the movement keys.   This panel is added to the view.
   */
  private OptionPanel _keysPnl = new OptionPanel("Movement Keys", 
                                                 new Object[]{IMovementKeys.ARROWSEND, IMovementKeys.WASDX});
  
  
  
  /**
   * An abstract factory used for manufacturing paint strategies.  The controller subclasses this using 
   * anonymous inner classes to produce custom factories at run time.
   */
  abstract interface IPaintFac { 
    /**
     * Instantiate an IPaintStrategy
     * @return An IPaintStrategy instance.
     */
    public abstract IPaintStrategy make();
  }
  
  /**
   * Constructor for the BallControl class
   * Used when run as an applet.
   */
  public BallControl (){
  }
  
  /**
   * Constructor for the BallControl class
   * Used when run as an application.  Called by main()
   * @param wl WindowListener that should call System.exit(0)
   */
  public BallControl (WindowListener wl)
  {
    _windowListener = wl;
    init();
  }
  
  /**
   * Initializes the system by instantiating the view and the model and all the necessary adapters.
   */
  public void init() {
    _view = new BallGUI(_windowListener, new IBallCreateAdapter() {
      public JPanel getConfigPanel(Object strategyFac, AbstractButton acceptBtn, AbstractButton cancelBtn){
        if(null!=strategyFac) return ((IStrategyFac) strategyFac).getConfigPanel(acceptBtn, cancelBtn);
        else return new JPanel() {
          public String toString() {
            return "Factory is null";
          }
        };
      }
      
      public void makeBall(Object strategyFac, Object paintFac) {
        if(null!=strategyFac && null!=paintFac) _model.loadBall(((IStrategyFac)strategyFac).make(), ((IPaintFac)paintFac).make());
      }
      
      public Object addStrategy(final String classname) {
        if(null==classname) return null;
        return new IStrategyFac() {
          public IUpdateStrategy make() {
            return (IUpdateStrategy) _model.loadStrategy(fixName("ballwar.model.strategy.",classname, "Strategy"), IUpdateStrategy.NullObject);          
          }
          
          public JPanel getConfigPanel(AbstractButton acceptBtn, AbstractButton cancelBtn) {
            return new JPanel() {
              {
                add(new JLabel("No configuration available"));
              }
              public String toString() {
                return classname+"Strategy configuration";
              }
            };
          }
          
          public String toString() {
            return classname.toString();
          }
        };
      }

      public Object addStrategyFac(final String classname) {
        if(null==classname) return null;
        return _model.loadStrategy(fixName("ballwar.model.strategy.", classname, "StrategyFac"), IStrategyFac.NullObject);
      }
      
      public Object addPaintStrategy(final String classname) {
        if(null==classname) return null;
        return new IPaintFac() {
          public IPaintStrategy make() {
            return (IPaintStrategy)_model.loadStrategy(fixName("ballwar.model.paint.", classname, "PaintStrategy"), IPaintStrategy.NullObject);          
          }
          public String toString() {
            return classname.toString();
          }
        };
      }
      
      public Object combineStrategies(final Object fac1, final Object fac2) {
        if(null==fac1 || null==fac2) return null;
        return new IStrategyFac() {
          public IUpdateStrategy make() {
            return new MultiStrategy(((IStrategyFac)fac1).make(),
                                     ((IStrategyFac)fac2).make());          
          }
          
          public JPanel getConfigPanel(final AbstractButton acceptBtn, final AbstractButton cancelBtn) {
            return new JPanel() {
              {
                // TODO: add tabs with each individual config on it
                JTabbedPane tp = new JTabbedPane();
                JPanel pnl = ((IStrategyFac)fac1).getConfigPanel(acceptBtn, cancelBtn);
                tp.add(pnl.toString(), pnl);
                pnl = ((IStrategyFac)fac2).getConfigPanel(acceptBtn, cancelBtn);
                tp.add(pnl.toString(), pnl);
                
                add(tp);
              }
              public String toString() {
                return fac1.toString()+"-"+fac2.toString()+"-Strategy configuration";
              }
            };
          }

          public String toString() {
            return fac1.toString()+"-"+fac2.toString();
          }
        };
      }
      
      public void makeSwitcher(Object pfac) {
        _model.loadSwitcherBall(((IPaintFac)pfac).make());
      }                           
    }, new IBallCtrlAdapter() {
      public  void paint(Graphics g) {
        _model.paint(g);
      }
      
      public void clearBalls(){
        _model.clearBalls();
      }
      
      public void switchStrategy(Object fac) {
        _model.setSwitcherStrategy(((IStrategyFac)fac).make());
      }
      
      public String getDisplayStr(String origStr) {
        return _model.getDisplayStr(origStr);
      }
    }, new IGameCtrlAdapter(){
      public Object makePlayer(String name) {
        return _model.makePlayer(name, (IMovementKeys)_keysPnl.removeSelectedItem());
      }
      public void start() {
      }
      public void end() {
      }
      public void pause() {
      }
    });
    _view.addComponent(_keysPnl);
    
    _model = new BallModel(new IViewCtrlAdapter() {
      public void update() {
        //System.out.println("view update");
        _view.update();
      }
      
      public Component getComponent() {
        return _view.getContainer();
      }
      
      public int getCanvasWidth() {
        return _view.getContainer().getWidth();
      }
      public int getCanvasHeight() {
        return _view.getContainer().getHeight();
      }
      public void addKeyCmd(String keyName, ILambda cmd) {
        _view.addKeyCmd(keyName, cmd);
      }
      public IPlayer getSelectedPlayer() {
        return (IPlayer)_view.getSelectedPlayer();
      }
      
    });
    
  }
  
  /**
   * Starts the systems running by validating the view and setting to be visible and starting the model.
   */
  public void start() {
    _view.validate ();
    _view.setVisible (true);
    _model.start ();
    
  }
  
  /**
   * Convenience method to add the rest of a strategy's classname plus the word "Strategy" at the end of the classname.
   * Assumes that the class is located in the ballwar.model.strategy package.
   * @param classname The classname, assuming that it is in the ballwar.model.strategy package and ends with the word "Strategy".
   * @return The complete, augmented classname.
   */
  private String fixName(String packageName, Object classname, String restOfName) {
//    return "ballwar.model.strategy."+classname.toString()+"Strategy";
    return packageName+classname.toString()+restOfName;
  }
  
  /**
   * Entry Point2DDouble to the program. Instantiates a BallControl object with a WindowListener (WindowAdapater) that calls System.exit(0) when the window is closed.
   */
  public static void main(String[] args)
  {
    (new BallControl(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        System.exit(0);
      }
      
    })).start();
  }
  
}

