package ballwar.model;

import util.*;
import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;
import java.util.*;

/**
 * An concrete circular ball that moves in a line with its given velocity and bounces off
 * the walls of a rectangularly shaped container.
 * A ball has a location, a radius, a mass, and environment and an update strategy. 
 */
public class Ball implements Observer {
  
  /**
   * The ball's environment
   * 
   */
  private IBallEnvironment _env = IBallEnvironment.NullEnv;
  
  /**
   * The present location of the center of the ball.
   */
  private Point2DDouble _location = new Point2DDouble(0,0);
  /**
   * The radius of the ball.
   */
  private int _radius = 0;
  
  
  private IBallMoveStrategy _moveStrategy = new IBallMoveStrategy() {
    private Point2DDouble _velocity = new Point2DDouble(0,0);
    
    public void move(Ball host) {
      double deltaTime = host.getEnv().getDeltaTime();
      host.getLocation().translate (deltaTime*_velocity.x, deltaTime*_velocity.y);
      bounce(host);
    }
    
    public Point2DDouble getVelocity() {
      return _velocity;
    }
    public void setVelocity(Point2DDouble v)
    {
      _velocity = v;
    }
    
    private void bounce(Ball host)
    {
      int _radius = host.getRadius();
      Point2DDouble _location = host.getLocation();
      int _minX = _radius;
      int _minY = _radius;
      int _maxX = host.getEnv().getWidth() - _radius;
      int _maxY = host.getEnv().getHeight() - _radius;
      if (_location.x < _minX)
      {
        _location.x = _minX; 
        _velocity.x = -_velocity.x;
      }
      else if(_location.x > _maxX)
      {
        _location.x = _maxX;
        _velocity.x = -_velocity.x;
      }
      if (_location.y < _minY)
      {
        _location.y = _minY;
        _velocity.y = -_velocity.y;
      }
      else if(_location.y > _maxY)
      {
        _location.y = _maxY;
        _velocity.y = -_velocity.y;
      }
    }
    
  };
  
  
  public IBallMoveStrategy getMoveStrategy() {
    return _moveStrategy;
  }
  
  public void setMoveStrategy(IBallMoveStrategy s) {
    _moveStrategy = s;
  }
  /**
   * The current velocity of the ball.   Given in terms of an X and Y increment per update interval.
   */
  //private Point2DDouble _velocity = new Point2DDouble(0,0);
  /**
   * The color of the ball.
   */
  private Color _color = Color.BLACK;
  
  /**
   * The mass of the ball
   */
  private double _mass = 1;
  
  /**
   * The update strategy used by the ball
   */
  private IUpdateStrategy _strategy = IUpdateStrategy.NullObject;
  
  /**
   * The paint strategy used by the ball
   */
  private IPaintStrategy _paintStrategy = IPaintStrategy.NullObject;
  
  private BallCmdSet _updateCmdSet = new BallCmdSet();
  
  /**
   * The command sent to the dispatcher to notify the other balls that they may have collided with this ball.
   * Students to add initialization here using anonymous inner class.
   * Should conform to these two specifications:
   * 1. No action taken if the target is the same ball (same instance) as this ball.
   * 2. If not the same ball, then this ball should checkCollide with the target ball, NOT the other way around.
   * This is because the algorithms assume that the ball doing the checking is the one that just moved.
   */
  /*
  private IBallCmd _collisionCmd = new IBallCmd() {
    public void apply(Ball target) {
      if(target == Ball.this) return;   // don't collide with self
      Ball.this.checkCollide(target);      
    }
  };
  */
  /**
   * Constructor for a ball.  Calls the init method of the given strategy to initialize the ball.
   * @param env The ball's environment
   * @param strategy The update strategy used by the ball.
   */
  public Ball(IBallEnvironment env, IUpdateStrategy strategy, IPaintStrategy pstrategy)
  {
    _env = env;
    setPaintStrategy(pstrategy);
    setStrategy(strategy);
    
    _env.getDispatcher().addObserver(this);
  }
  
  /**
   * The update method called by the main ball Dispatcher to notify all the balls to perform the given command.
   * The given command is executed.
   * @param o The Dispatcher that set the update request.
   * @param cmd The IBallCmd that will be run.
   */
  public void update(Observable o, Object cmd) {
    ((IBallCmd)cmd).apply(this);
  }
  
  public void addUpdateCmd(IBallCmd cmd) {
    _updateCmdSet.addCmd(cmd);
  }
  
  /**
   * Updates the state of the ball by calling the strategy's updateState(), moving the ball as per its velocity, bouncing off the walls if necessary,
   * and notifying all other balls that it may have collided with them.  Finally, the ball is painted onto the given Graphics object.
   * @param g The Graphics object to paint on.
   */
  public void updateState(Graphics g)
  {
    //bounce();
//    _env.getDispatcher().notifyAll(_collisionCmd);
    _strategy.updateState(this); // update this ball's state using the strategy.
    // Moves the ball by the X and Y increments specified by the velocity property.
//    paint(g);
//    bounce();
    //move();
    // Students:  At this point, get the dispatcher from the environment and 
    // tell it to notifyAll the balls with your collision command.  
//    _env.getDispatcher().notifyAll(_collisionCmd);
    
//    paint(g);
    
  }
  
  public void performUpdate() {
    _updateCmdSet.apply(this);
    _updateCmdSet.clear();
    move();
  }
  
  private void move() {
    _moveStrategy.move(this);
//    double deltaTime = _env.getDeltaTime();
//    _location.translate (deltaTime*_velocity.x, deltaTime*_velocity.y);
  }
  
  /**
   * Checks if the ball needs to bounce off the wall of its container.  
   * Performs a perfectly elastic bounce, moving the ball back completely onto the 
   * drawing area if necessary.
   */
  /*
  public void bounce()
  {
    int _minX = _radius;
    int _minY = _radius;
    int _maxX = _env.getWidth() - _radius;
    int _maxY = _env.getHeight() - _radius;
    if (_location.x < _minX)
    {
      _location.x = _minX; 
      _velocity.x = -_velocity.x;
    }
    else if(_location.x > _maxX)
    {
      _location.x = _maxX;
      _velocity.x = -_velocity.x;
    }
    if (_location.y < _minY)
    {
      _location.y = _minY;
      _velocity.y = -_velocity.y;
    }
    else if(_location.y > _maxY)
    {
      _location.y = _maxY;
      _velocity.y = -_velocity.y;
    }
  }
  */
  /**
   * Sets the center of the ball to a new location.
   * @param location the new center.
   */
  public void setLocation(Point2DDouble location)
  {
    _location = location;
  }
  
  /**
   * Returns the current center of this Ball.
   * @return a Point2DDouble representing the center of this Ball.
   */
  public Point2DDouble getLocation() {
    return(_location);
  }
  
  /**
   * Sets the radius of this Ball to a new radius.
   * @param radius 
   */
  public void setRadius(int radius) {
    _radius = radius;
  }
  
  /**
   * Returns the current radius for this Ball.
   * @return the radius of this Ball.
   */
  public int getRadius()
  {
    return(_radius);
  }
  
  /**
   * Sets the velocity of this Ball to a new velocity.
   * @param velocity the new velocity for this Ball.
   */
  public void setVelocity(Point2DDouble velocity)
  {
    _moveStrategy.setVelocity(velocity);
    //_velocity = velocity;
  }
  
  /**
   * Returns the current velocity of this Ball.
   * @return the current velocity of this Ball.
   */
  public Point2DDouble getVelocity()
  {
    return _moveStrategy.getVelocity();
    //return _velocity;
  }
  
  /**
   * Sets the color of this Ball to a given color.
   * @param color the new color for this Ball.
   */
  public void setColor(Color color)
  {
    _color = color;
  }
  
  /**
   * Returns the current color of this Ball.
   * @return the color of this Ball.
   */
  public Color getColor()
  {
    return(_color);
  }
  
  /**
   *  Returns the current strategy of this Ball
   * @return the IUpdateStrategy currently in use.
   */
  public IUpdateStrategy getStrategy() {
    return _strategy;
  }
  
  /**
   * Sets the current paint strategy of this Ball to the given strategy
   * @param pstrategy The new IPaintStrategy to use.
   */
  public void setPaintStrategy(IPaintStrategy pstrategy) {
    _paintStrategy = pstrategy;
    _paintStrategy.init(this);
  }
  
  /**
   *  Returns the current paint strategy of this Ball
   * @return the IPaintStrategy currently in use.
   */
  public IPaintStrategy getPaintStrategy() {
    return _paintStrategy;
  }
  
  /**
   * Sets the current update strategy of this Ball to the given strategy
   * @param strategy The new IUpdateStrategy to use.
   */
  public void setStrategy(IUpdateStrategy strategy) {
    _strategy = strategy;
    _strategy.init(this);
  }
  
  /**
   * Paints the image of the ball onto a Graphics object 
   * using the current paint strategy.
   * @param g a Graphics object.
   */
  public void paint(Graphics g)  {
    _paintStrategy.paint(g, this);
  }
  /**
   * "Kills" this ball by delegating the request to the strategy.
   */
  public void kill() {
    _strategy.kill(this);
  }
  
  /**
   * Kills th ball by removing it from the dispatcher.   This will effectively remove the ball from the system.
   */
  public void doKill() {
    _env.getDispatcher().deleteObserver(this);
  }
  
  /**
   * Get the ball's environment
   * @return the ball's environment
   */
  public IBallEnvironment getEnv() {
    return _env;
  }
  
  /**
   * Get the ball's mass
   * @return The ball's mass
   */
  public double getMass() {
    return _mass;
  }
  /**
   * Sets the ball's mass
   * @param mass The mass value.  Must be a positive, non-zero number.
   */
  public void setMass(double mass) {
    _mass = mass;
  }
  
  /**
   * Checks if this ball has collided with the given target ball.
   * Calls collidedWith() if it has collided.
   * @param aBall The target ball.
   */
  /*
  public void checkCollide(Ball aBall) {
    double distance = _location.distance(aBall.getLocation());
    double minSeparation =_radius + aBall.getRadius(); 
    if( minSeparation > distance) {
      collidedWith(aBall, distance, minSeparation-distance);
    }
  }
  */
  /**
   * Percentage amount to "nudge" collided balls away from each other
   * beyond their minimum distance.   This helps prevent re-colliding.
   */
  //private static double _Nudge = 1.01;
  
  /**
   * Called when this ball has moved and collided with the target ball.   Recalculates the velocity assuming
   * a perfectly elastic collision between to non-zero mass circular objects.
   * The target ball's collidedWithHelp() method is called and 
   * then the strategy's updateCollision() method is called.
   * @param aBall The target ball
   * @param distance The actual, current separation distance between the two balls' centers.
   * @param deltaR The difference in distance between the minimum allowed separation of the two balls and the actual separation.
   */
  /*
  public void collidedWith(Ball aBall, double distance, double deltaR) {
    double mass = getMass();
    // Reduced mass = (m1*m2)/(m1+m2)
    double mr;
    if(mass == Double.POSITIVE_INFINITY) mr = aBall.getMass();
    else mr = mass/(1.0+(mass/aBall.getMass()));
    // normal direction
    double nx = ((double)(_location.x - aBall.getLocation().x))/distance;
    double ny = ((double)(_location.y - aBall.getLocation().y))/distance;
    // delta velocity in normal direction  (va-vb).n
    double dvn = (_velocity.x - aBall.getVelocity().x)*nx + (_velocity.y - aBall.getVelocity().y)*ny; 
    double impx = 2.0*mr*dvn*nx;
    double impy = 2.0*mr*dvn*ny;
    _velocity.translate(-impx/mass, -impy/mass);
//    _velocity.translate((int)Math.round(-impx/mass), (int)Math.round(-impy/mass));
//    _location.translate((int)Math.round(_Nudge*nx*deltaR), (int)Math.round(_Nudge*ny*deltaR));
    _location.translate(_Nudge*nx*deltaR, _Nudge*ny*deltaR);
    
//    aBall.collidedWithHelp(this, -impx, -impy); 
    _strategy.updateCollision(this, aBall);
  }
  */
  /**
   * Called when another ball has moved an collided with this target ball.
   * Adjusts this ball's velocity as per the given momentum impulse parameters.
   * This ball's strategy's collisionUpdate() method is called after the velocity is modified.
   * @param aBall The ball that moved and collided with this ball.
   * @param impx The x-component of the collision impulse
   * @param impy The y-component of the collision impulse
   */
  /*
  public void collidedWithHelp(Ball aBall, double impx, double impy) {
    _velocity.translate(-impx/_mass, -impy/_mass);
//    _velocity.translate((int)Math.round(-impx/_mass), (int)Math.round(-impy/_mass));
    _strategy.updateCollision(this, aBall);
  }
  */
}


