package ballwar.model;

import util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;


/**
 * Represents the main "model" of the system that coordinates the non-GUI processing.
 * 
 */
public class BallModel implements IBallEnvironment {
  
  /**
   * The adapter to the view.
   */
  private IViewCtrlAdapter _vCtrlA;
  
  /**
   * The dispatcher used to communicate with all the balls.
   */
  private Dispatcher _dispatcher = new Dispatcher();
  /**
   * The SwitcherStrategy object to use in switcher balls.
   */
  private ballwar.model.strategy.SwitcherStrategy _switcher = new ballwar.model.strategy.SwitcherStrategy();
  /**
   * The command used when the balls are told to update their states and paint.
   */
  private IBallCmd _updateStateCmd = new IBallCmd() {
    public void apply(Ball context) {
      context.updateState(_graphics);
    }
  };
  
  private IBallCmd _performUpdateCmd = new IBallCmd() {
    public void apply(Ball context) {
      context.performUpdate();
    }
  };
  
  private IBallCmd _paintCmd = new IBallCmd() {
    public void apply(Ball context) {
      context.paint(_graphics);
    }
  };
  
  
  /**
   * Variable used to hold the current Graphics object 
   * upon which the balls will next paint.
   */
  private Graphics _graphics;
  /**
   * The time in milliseconds between updates to the balls.
   */
  private int updatesPerPaint = 50;
  private double _deltaTime = 1.0/updatesPerPaint;
  private int _paintInterval = 50; 
  private int _timeSlice = (int) Math.round(_paintInterval/updatesPerPaint);//50;
  /**
   * A timer object that is the main clock that drives the simulation.
   * Every _timeSlice interval the view is asked to update, which
   * in turn will ask us to do the same.
   */
  private Timer _paintTimer = new  Timer (_paintInterval, new ActionListener ()                                       {
    public void actionPerformed (ActionEvent e) {
      _vCtrlA.update();
    }
  });
  
// private Timer _updateTimer = new  Timer (_timeSlice, new ActionListener ()                                       {
//    public void actionPerformed (ActionEvent e) {
//      CollisionCmd.Singleton.reset();
//     // _dispatcher.notifyAll(CollisionCmd.Singleton);
//      _dispatcher.notifyAll(_updateStateCmd);
//    }
//  });
  
  private java.util.TimerTask _updateTask = new java.util.TimerTask(){
    public void run(){
      _dispatcher.notifyAll(_updateStateCmd);
      _dispatcher.notifyAll(_performUpdateCmd);      
    }
  };
  
  private java.util.Timer _updateTimer = new java.util.Timer();
  
  /**
   * The initial, default command used to generate the initial display string.
   */
  private ILambda _displayCmdOrig = new ILambda() {
    public Object apply(Object o) {
      return o+"Ready    ";
    }
  };
  /**
   * The current command used to generate the display string.
   */
  private ILambda _displayCmd = _displayCmdOrig;
  
  /**
   * Constructor for the BallControl class
   * Instantiates the BallGUI frame with the required ILambdas for 
   * making a ball, clearing the balls from the screen and for 
   * painting the balls onto the Container
   * Starts the timer up.
   */
  public BallModel(IViewCtrlAdapter vCtrlA)
  {
    _vCtrlA = vCtrlA;
  }
  
  /**
   * Get the dispatcher used to notify all the balls to execute a supplied command.
   * @return The system's dispatcher.
   */
  public Dispatcher getDispatcher() {
    return _dispatcher;
  }
  
  
  public Component getComponent() {
    return _vCtrlA.getComponent();
  }  
  
  /**
   * Get's the width of the ball's environment.
   * @return The width of the environment in pixels
   */
  public int getWidth() {
    return _vCtrlA.getCanvasWidth();
  }
  /**
   * Gets the height of the ball's environment
   * @return The height of the ball's environment in pixels.
   */
  public int getHeight() {
    return _vCtrlA.getCanvasHeight();
  }
  
  public double getDeltaTime() {
    return _deltaTime;
  }
  
  /**
   * Adds a display command (ILambda) used to display messages onto the view.
   * The display command is assumed to take a default string and return a string.  
   * The ILambda's apply() output is concatenated onto any previously existing 
   * command's output separated by a space.    The command is called whenever 
   * the view requests that the display string be updated (ie. when getDisplayStr() is called).   It is not required that the
   * command's apply method use its input parameter.
   * @param cmd The ILambda used to generate the display string.
   */
  public void addDisplayCmd(final ILambda cmd) {
    _displayCmd = new ILambda(){
      private ILambda _oldCmd = _displayCmd;
      public Object apply(Object o){
        return _oldCmd.apply(o)+ " "+ cmd.apply(o);
      }
    };
  }
  
  /**
   * Called by the view to get the current display string.   Calls the concatenated display commands
   * added with the addDisplayCmd method.  
   * @param origStr The string handed to the display commands as their input parameter.
   * @return The complete  display string.
   */
  public String getDisplayStr(String origStr) {
    return (String)_displayCmd.apply(origStr);
  }
  
  /**
   * @param keyName 
   * @param cmd 
   */
  public void addKeyCmd(String keyName, ILambda cmd) {
    _vCtrlA.addKeyCmd(keyName, cmd);
  }
  
  /**
   * @param strategy 
   * @return A new instance of a Ball.
   */
  public Ball loadBall(IUpdateStrategy strategy, IPaintStrategy pstrategy) {
    return new Ball(this, strategy, pstrategy);
  }
  
  /**
   * Uses dynamic class loading to load and instantiate a specified class, given
   * its full class name.
   * @param className full classname of the strategy 
   * @param defaultObject The default object instance to return if the given class cannot be instantiated.  Usually specified to be an instance of the class's null object value.
   * @return An instance of the given class, typed as Object
   */
  public Object loadStrategy(String className, Object defaultObject) {
    try {
      java.lang.reflect.Constructor c =  Class.forName(className).getConstructor();
      return c.newInstance();
    }
    catch(Exception ex) {
      System.err.println("Class "+className+" not found!  Using default object instead: "+defaultObject);
      return defaultObject;
    }
  }
  
  /**
   * Tells the model to have all the balls to update their states and paint themselves onto the supplied Graphics object.
   *  
   * @param g The Graphics object to paint on.
   */
  public void paint(Graphics g) {
    _graphics = g;
    _dispatcher.notifyAll(_paintCmd);
  }
  
  /**
   * Clears all the balls from the dispatcher and resets the display string to its original state.
   */
  public void clearBalls() {
    _dispatcher.deleteObservers();
    _displayCmd = _displayCmdOrig;
  }
  
  /**
   * Sets the strategy used by any switcher balls.
   * @param strategy The new strategy to use.
   */
  public void setSwitcherStrategy(IUpdateStrategy strategy){
    _switcher.setStrategy(strategy);
  }
  
  /**
   * Loads a ball with a switcher strategy into the system.
   */
  public void loadSwitcherBall(IPaintStrategy pstrategy) {
    loadBall(_switcher, pstrategy);
  }
  
  /**
   * Starts the model running by starting the internal timer.
   */
  public void start() {
    _paintTimer.start();
    //_updateTimer.start();
    _updateTimer.scheduleAtFixedRate(_updateTask, 0, _timeSlice);
  }
  
  /**
   * Makes a play8er object given a name and a set of movement keys.   
   * If the name is an empty string, the NullPlayer is returned.
   * Also adds the player's score to display string.
   * @param name The name of the new player
   * @param keys The set of movement keys associated with this player.
   * @return An IPlayer instance.
   */
  public IPlayer makePlayer(final String name, final IMovementKeys keys) { 
    if(""==name) return IPlayer.NullPlayer;
    else 
      return new  IPlayer() {
      //initialization block for anonymous inner class.  Runs like a constructor
      // Has no name b/c anonymous inner class has no name!
      {
        addDisplayCmd(new ILambda() {
          public Object apply(Object o) { 
            return  name+"'s Score = "+_score+"   ";
          }
        });
      }
      private int _score;
      private IMovementKeys _keys = keys;
      public int getScore() {
        return _score;
      }
      public void setScore(int score){
        _score = score;
      }
      
      public String getName() {
        return name;
      }
      public IMovementKeys getMovementKeys() {
        return _keys;
      }
      public void setMovementKeys(IMovementKeys keys) {
        _keys = keys;
      }
      public String toString() {
        return name;
      }
    };
  }
  
  /**
   * Registers a set of movement keys, as defined by an IMovementKeys object,
   * with the view.   Each key is associated with its corresponding action in the IMoveable object.
   * @param keys The set of movement keys to register
   * @param m The IMoveable object to associate with those keys.
   */
  public void registerMovementKeys(IMovementKeys keys, final IMoveable m) {
    
    _vCtrlA.addKeyCmd(keys.getLeftKey(), new ILambda() {
      public Object apply(Object o) {
        m.moveLeft();
        return null;
      }
    });
    _vCtrlA.addKeyCmd(keys.getRightKey(), new ILambda() {
      public Object apply(Object o) {
        m.moveRight();
        return null;
      }
    });
    _vCtrlA.addKeyCmd(keys.getUpKey(), new ILambda() {
      public Object apply(Object o) {
        m.moveUp();
        return null;
      }
    });
    _vCtrlA.addKeyCmd(keys.getDownKey(), new ILambda() {
      public Object apply(Object o) {
        m.moveDown();
        return null;
      }
    });
    _vCtrlA.addKeyCmd(keys.getStopKey(), new ILambda() {
      public Object apply(Object o) {
        m.stop();
        return null;
      }
    });
  }
  
  /**
   * Get the currently selected player from the view.
   * @return The currently selectd player
   */
  public IPlayer getSelectedPlayer() {
    return _vCtrlA.getSelectedPlayer();
  }
  
}

