package ballwar.model;

import util.*;

import ballwar.controller.*;

import util.*;
import java.awt.*;

/**
 * An interface that represents the environment in which the ball is operating.
 */
public abstract interface IBallEnvironment {
  /**
  * Represents a null environment that does nothing.
  */
 public IBallEnvironment NullEnv = new IBallEnvironment() {
    public Dispatcher getDispatcher() {
      return null;
    }
    public void addKeyCmd(String keyName, ILambda cmd) {
    }
    public void addDisplayCmd(ILambda cmd) {
    }
    public Component getComponent() {
      return null;
    }
    public int getWidth() {
      return 0;
    }
    public int getHeight() {
      return 0;
    }
    
    public double getDeltaTime(){
      return 1.0;
    }
    public IPlayer getSelectedPlayer() {
      return IPlayer.NullPlayer;
    }
    public void registerMovementKeys(IMovementKeys keys, IMoveable m) {}
  };
  /**
  * Get the ball dispatcher used by the system.   Used when a ball wants to communicate to all the other balls, e.g. for collisions.
  * @return The system ball dispatcher
  */
 public abstract Dispatcher getDispatcher();
  /**
  * Associates a key with a behavior (command).  
  * When the user presses the given key, the ILambda command will be run.
  * The parameter passed to the command will be the key name.   
  * The return value will be discarded.
  * @param keyName The name of the key, as defined by the Java "virtual keys":   VK_XXX  (The "VK_" is omitted).   See the Java documentation under "KeyEvent"
  * @param cmd The ILambda command that will be run when the key is pressed.
  */
 public abstract void addKeyCmd(String keyName, ILambda cmd);
  /**
  * Adds a command that will be run to whenever the view requests a display string to be displayed.   
  * The output of the ILambda.apply must be a String given a String input, where the input is the original string
  * on the display.   The output is concatenated with all other display commands in the system.
  * @param cmd The command used to generate part of the total display string.
  */
 public abstract void addDisplayCmd(ILambda cmd);
  
 /**
  * Gets the Component that the Balls will be painted on.  Note that 
  * a Component class is also implements the ImageObserver interface.
  * @return The Component object upon which the Balls will be painted. 
  */
 public abstract Component getComponent();
 /**
  * Get the width of the ball's environment.    This is not the same as the width of the entire BallWar frame, but rather
  * simply the width of the graphics area in which the balls are moving.
  * @return The width of the environment in pixels.
  */
 public abstract int getWidth();
  /**
  * Get the height of the ball's environment.    This is not the same as the height of the entire BallWar frame, but rather
  * simply the height of the graphics area in which the balls are moving.
  * @return The height  of the environment in pixels.
  */
 public abstract int getHeight();
  /**
  * Get the currently selected player.   Note that in general, for a ball or strategy to maintain a 
  * connection to a particular player, they must save their own reference and not rely on the 
  * currently selected player.   Usually this method is only called during an initialization phase.
  * @return The currently  selected player.
  */
 
 public abstract double getDeltaTime();
 
 public abstract IPlayer getSelectedPlayer();
  /**
  * Takes an IMovementKeys object which defines a set of movement keys and 
  * associates the pressing of each key with its respective behavior in the given IMoveable
  * object.      
  * @param keys The IMovementKeys object that defines a set of movement keys.
  * @param m The IMoveable object that is to be associated with the key set.
  */
 public abstract void registerMovementKeys(IMovementKeys keys, IMoveable m);
}