package ballwar.model;




/**
 * Interface that represents a strategy used by a Ball when asked to perform variant operations associated 
 * with initialization, updating its state, collisions and destruction.
 */
public interface IUpdateStrategy {
  /**
  * Updates the state of the given ball in response to the periodic repainting of the ball as a part of its animation..  
  * Used to modify the position, velocity, color, etc as the ball is moving.
  * @param context The ball to update.
  */
 public abstract void updateState(Ball context);
  /**
  * updates the state of the ball in response to a collision between the two balls.  
  * It is assumed that the context ball is the one that just moved and collided with 
  * the target ball.
  * @param context The ball that just moved and collided with the target ball.
  * @param target The ball that was not moving and was collided by the context.
  */
 public abstract void updateCollision(Ball context, Ball target);
  /**
  * Initializes the given ball.   Called once by the ball when the strategy is first loaded.
  * @param context The ball to initialize.
  */
 public abstract void init(Ball context);
  /**
  * Processes a "killing" of a ball. Called when a ball's kill() method is called.   
  * A ball will only be removed from the system if its doKill() method is called, perhaps 
  * from the strategy's kill() method.
  * @param context The ball who's kill method was called.
  */
 public abstract void kill(Ball context);
 
 /**
  * Singleton null object instance of IUpdateStrategy that 
  * does nothing.
  */
 public static final IUpdateStrategy NullObject = new IUpdateStrategy() {
   public void updateState(Ball context) {}
   public void updateCollision(Ball context, Ball target) {}
   public void init(Ball context) {}
   public void kill(Ball context) {}
 };
}