package ballwar.model.paint;

import util.*;
import ballwar.model.*;
import java.awt.*;
import java.awt.geom.*;

/**
 * Abstract class that provides default behavior for subclasses
 * that will decorate another IPaintStrategy to add functionality
 * to that strategy.   All this class's methods do is to simply delegate 
 * to the decoree.  A subclass should override one or more methods, 
 * adding additional processing either before or after delegating to the 
 * decoree.
 * Note that this class cannot be used by the BallWar system directly 
 * as it lacks a no-parameter constructor.
 */
public abstract class ADecoratorPaintStrategy implements IPaintStrategy {

  /**
   * The "decoree" paint strategy whose methods are being augmented
   * by this decorator paint strategy.
   */
  private IPaintStrategy decoree;

  /**
   * Constructor that takes the decoree paint strategy
   * @param decoree  The paint strategy that is to be decorated
   */
  public ADecoratorPaintStrategy(IPaintStrategy decoree) {
    this.decoree = decoree;
  }
  
  /**
   * Default behavior is to simply delegate to the decoree's paint method
   * @param g The graphics context which is passed to the decoree
   * @param host The host Ball which is passed to the decoree
   */
  public void paint(Graphics g, Ball host) {
    decoree.paint(g, host);
  }
    

  /**
   * Default behavior is to simply delegate to the decoree's paintXfrm method
   * @param g The graphics context which is passed to the decoree
   * @param host The host Ball which is passed to the decoree.
   * @param at The affine transform which is passed to the decoree.
   */
  public void paintXfrm(Graphics g, Ball host, AffineTransform at) {
    decoree.paintXfrm(g, host, at);
  }

  /**
   * Default behavior is to simply delegate to the decoree's init method
   * @param host The host Ball which is passed to the decoree.
   */
  public void init(Ball host){
    decoree.init(host);
  }
 
  
}