package ballwar.model.paint;

import ballwar.model.*;
import java.awt.*;
import java.awt.geom.*;

/**
 * An abstract paint strategy that provides default behavior for its subclasses.
 */
public abstract class APaintStrategy implements IPaintStrategy {
  
  /**
   * The affine transform used by this paint strategy to translate, scale and rotate the image.
   */ 
  protected AffineTransform at;
  
  /**
   * Constructor that initializes the strategy with an affine transform
   * @param at The AffineTransform for this paint strategy to use.
   */
  public APaintStrategy(AffineTransform at) {
    this.at = at;
  }
  
  /**
   * Paints on the given graphics context using the color, scale and 
   * direction provided by the host.  This is done by setting up the 
   * AffineTransform to rotate then scale then translate.
   * Calls paintXfrm to actually perform the painting, using the set up transform.
   * Calls paintCfg just before calling paintXfrm
   * param g The Graphics context that will be paint on
   * param host The host Ball that the required information will be pulled from.
   */
  public void paint(Graphics g, Ball host) {
    double scale = host.getRadius();
    at.setToTranslation(host.getLocation().x, host.getLocation().y);
    at.scale(scale, scale);
    at.rotate(Math.atan2(host.getVelocity().y, host.getVelocity().x));
    g.setColor(host.getColor());    
    paintCfg(g, host);
    paintXfrm(g, host, at);
  }
  
  /**
   * By default, do nothing for initialization.
   */
  public void init(Ball context){
  };

  
  /**
   * Used for doing additional configurations.  All additional transformations 
   * applied to the existing internal affine transform will be performed before
   * those set up by the paint method.  This method is designed to be overridden by 
   * the subclass.  By default the behavior is a no-op.
   * @param g The Graphics context that will be drawn upon.
   * @param host The Ball to be painted.
   */
  protected void paintCfg(Graphics g, Ball host){
  }
  
}