package ballwar.model.paint;

import ballwar.model.*;
import java.awt.*;
import java.awt.geom.*;

/**
 * Concrete paint strategy that cycles through a sequence of paint 
 * strategies, painting one per paint update.
 * Note: This paint strategy cannot be directly instantiated by the 
 * BallWar system because it does not have a no-parameter constructor.
 */
public class AnimatePaintStrategy extends APaintStrategy {
  
  /** 
   * The paint strategies to cycle through
   */
  private IPaintStrategy paintStrats[];
  
  /**
   * Constructor that instantiates an AffineTransform for internal use.
   * @param pstrats vararg input of the paint strategies to cycle through, in order.
   */
  public AnimatePaintStrategy(IPaintStrategy... pstrats){
    this(new AffineTransform(), pstrats);
  }
  
  /**
   * Constructor that uses the supplied AffineTransform for internal use.
   * @param at The AffineTransform to use.
   * @param pstrats vararg input of the paint strategies to cycle through, in order.
   */
  public AnimatePaintStrategy(AffineTransform at, IPaintStrategy... pstrats){
    super(at);
    paintStrats = pstrats;
  }
  
  /**
   * The counter that keeps track of which paint strategy to use next.
   */
  private int count = 0;
  
  
  /**
   * Paints the currently indexed paint strategy on the 
   * given Graphics context using the 
   * supplied AffineTransform. Called by the inherited paint method.
   * @param g The Graphics context to paint upon.
   * @param host The host Ball
   * @param at The AffineTransform to use
   */
  public void paintXfrm(Graphics g, Ball host, AffineTransform at){
    paintStrats[count++].paintXfrm(g, host, at);
    if(count>=paintStrats.length) count = 0;
  }
}