package ballwar.model.paint;

import util.*;
import ballwar.model.*;
import java.awt.*;
import java.awt.geom.*;

/**
 * Paint strategy that paints a filled circle with the Ball's radius.
 * This functionality is duplicated by the EllipsePaintStrategy.
 * The class demonstrates a direct implementation of IPaintStrategy.
 */
public class BallPaintStrategy implements IPaintStrategy {

  /**
   * Unit sized circle used as a prototype
   */
  private Ellipse2D.Double ball;
  
  /**
   * The AffineTransformed used for internal calculations.
   */
  private AffineTransform at;
  
  /** 
   * No parameter constructor that creates a 1 pixel radius ball 
   * at the origin.  Instantiates a new AffineTransform for internal 
   * use.
   */
  public BallPaintStrategy() {
    this(new AffineTransform(), 0, 0, 1, 1);
  }

  /** 
   * Constructor that allows one to create the prototype ball of arbitrary 
   * dimension and location.   The AffineTransform is externally supplied.
   * @param at The AffineTransform to use for internal calculations
   * @param x floating Point2DDouble x-coordinate of center of circle
   * @param y floating Point2DDouble y-coordinate of center of circle
   * @param width floating Point2DDouble x-radius of the circle (ellipse)
   * @param height floating Point2DDouble y-radius of the circle (ellipse)
   */
  public BallPaintStrategy(AffineTransform at, double x, double y, double width, double height) {
   this.at = at;
   ball = new Ellipse2D.Double(x-width, y-height, width*2, height*2);
  }
  
  /**
   * Paints on the given graphics context using the color, scale and 
   * direction provided by the host.  This is done by setting up the 
   * AffineTransform to scale then translate.
   * Calls paintXfrm to actually perform the painting, using the set up transform.
   * Calls paintCfg just before calling paintXfrm
   * param g The Graphics context that will be paint on
   * param host The host Ball that the required information will be pulled from.
   */
  public void paint(Graphics g, Ball host) {
      double scale = host.getRadius();
      at.setToTranslation(host.getLocation().x, host.getLocation().y);
      at.scale(scale, scale);
      g.setColor(host.getColor());
      paintXfrm(g, host, at);
  }
  
  /**
   * Paints a transformed circle, as per the given AffineTransform
   * Uses color already set in Graphics context
   * @param g The Graphics context to paint on
   * @param host The Ball host 
   * @param at the AffineTransform to use
   */
  public void paintXfrm(Graphics g, Ball host, AffineTransform at) {
      ((Graphics2D) g).fill(at.createTransformedShape(ball));
  }
  
  /**
   * By default, do nothing for initialization.
   */
  public void init(Ball context){
  };
  
}