package ballwar.model.paint;

import util.*;
import ballwar.model.*;
import java.awt.*;
import javax.swing.*;
import java.awt.geom.*;
import java.awt.image.*;

/**
 * Paint strategy that paints an image from a file, scaled to the host 
 * Ball's radius.
 * Note that this class cannot be used by the BallWar system directly as it 
 * is missing a no-parameter constructor.
 */
public class ImagePaintStrategy extends APaintStrategy {
  /**
   * ImageObserver needed for some image operations
   */
  private ImageObserver imageObs;
  
  /**
   * The image to paint
   */
  private Image image;
  
  /**
   * Number of times larger than unit size of image 
   */
  private double scaleFactor = 1.0; 
  
  /**
   * Constructor that takes the image filename and fill factor.   
   * An AffineTransform is instantiated for internal use.
   * GIF, JPG, PNG and any other file that can be loaded by an ImageIcon
   * object can be used, including animated GIF's as the above files are.
   * @param filename The filename of the image file to use.
   * @param fillFactor The ratio of the desired average radius of the image to the actual average of the image's width and height.
   */
  public ImagePaintStrategy(String filename, double fillFactor) {
    this(new AffineTransform(), filename, fillFactor);
  }
  
  /**
   * Constructor that takes an external AffineTransform, the filename of the
   * image to paint and a fill factor of the image.
   * @param at The AffineTransform to use internally.
   * @param filename The filename of the image file to use.
   * @param fillFactor The ratio of the desired average radius of the image to the actual average of the image's width and height.
   */
  public ImagePaintStrategy(AffineTransform at, String filename, double fillFactor) {
    super(at);
    // The pathname is with respect to the current package, ie. ballwar.model.paint
    // Note: DrJava does not automatically include non-Java files when it 
    // makes jar files, so they must be added manually afterwards.
    image = new ImageIcon(this.getClass().getResource(filename)).getImage();
// If you are definitely only reading from the disk and not a jar file
// then the following simpler line can be used. The pathname will then 
// be relative to the working directory.
//    image = new ImageIcon(filename).getImage();
    scaleFactor = fillFactor*(image.getWidth(imageObs)+image.getHeight(imageObs))/2.0;
  }
  
  /**
   * Augments the inhereited paint method to add
   * the scaling of the image by the scaleFactor 
   * and translation to the center of the image.
   */
  protected void paintCfg(Graphics g, Ball host) {
    at.scale(1.0/scaleFactor, 1.0/scaleFactor);
    at.translate(-image.getWidth(imageObs)/2.0, -image.getHeight(imageObs)/2.0);
  }
  
  /**
   * Draws the image on the given Graphics context using the given 
   * affine transform.
   * @param g The graphics context to paint on
   * @param host The host Ball
   * @param at The AffineTransform to use.
   */
  public void paintXfrm(Graphics g, Ball host, AffineTransform at){
    ((Graphics2D)g).drawImage(image, at, host.getEnv().getComponent());
  }
  
  /**
   * Initializes the internal ImageObserver reference from the host Ball
   * @param host The host Ball
   */
  public void init(Ball host) {
    imageObs = host.getEnv().getComponent();
  }
}