package ballwar.model.paint;

import util.*;
import ballwar.model.*;
import java.awt.*;
import java.awt.geom.*;

/**
 * A composite design pattern exension of APaintStrategy that 
 * paints a set of paint strategies.
 * Note: This paint strategy cannot be used directly by the BallWar
 * system because it lacks a no-parameter constructor.
 */
public abstract class MultiPaintStrategy extends APaintStrategy {

  /**
   * The set of paint strategies to paint
   */
  private IPaintStrategy[] pstrats;

  /**
   * Constructor that takes the paint strategies that 
   * will part of the composite.
   * An AffineTransform is instantiated for internal use.
   * @param pstrats Vararg parameter that are the paint strategies that will make up the composite.
   */
  public MultiPaintStrategy(IPaintStrategy... pstrats) {
    this(new AffineTransform(), pstrats);
  }
  
  
  /**
   * Constructor that takes the paint strategies that 
   * will part of the composite.
   * An external AffineTransform is supplied for internal use.
   * @param at The AffineTransform to use.
   * @param pstrats Vararg parameter that are the paint strategies that will make up the composite.
   */
  public MultiPaintStrategy(AffineTransform at, IPaintStrategy... pstrats) {
    super(at);
    this.pstrats = pstrats;
  }
  
  /**
   * Delegates to all the IPaintStrategies in the composite.
   * Paints using given Graphics context using the 
   * supplied AffineTransform. Called by the inherited paint method.
   * @param g The Graphics context to paint upon.
   * @param host The host Ball
   * @param at The AffineTransform to use
   */
  public void paintXfrm(Graphics g, Ball host, AffineTransform at) {
    for(IPaintStrategy ps : pstrats) {
      ps.paintXfrm(g, host, at);
    }
  }

  
  /**
   * Delegates to all the IPaintStrategies in the composite.
   * Used to initialize the paint strategies.
   * @param host The host Ball
   */
  public void init(Ball host){
    for(IPaintStrategy ps : pstrats) {
      ps.init(host);
    }
  }
 
  
}