package ballwar.model.paint;
import java.awt.*;
import java.awt.geom.*;
import util.*;

/**
 * Concrete IShapeFactory that provides the invariant behavior to 
 * instantiate a Shape that is a Polygon.   
 * This class can be instantiated and used simply by supplying the 
 * desired points in its constructor, or sub-classed an the constructor
 * overridden.
 * Note that this class cannot be used directly by the BallWar system 
 * because it does not have a no-parameter constructor.
 */
public class PolygonFactory implements IShapeFactory {
  /**
   * The Polygon shape to use as the prototype.
   */
  private Polygon poly = new Polygon();
  
  /**
   * The AffineTransform used for internal calculations
   */
  private AffineTransform at;
  
  /**
   * Scale factor that scales the integer Point2DDouble-defined Polygon
   * to a unit size, which requires doubles.   
   * The value of scaleFactor is The ratio of the desired unit size 
   * to the defined size of the prototype Polygon.
   */
  private double scaleFactor = 1.0;
  
  /**
   * Constructor that uses an externally defined AffineTransform for internal
   * use plus takes the defining points of the prototype Polygon and 
   * a scale factor to scale the given points to the desired unit size. 
   * Since Polygons require Point2DDouble objects for their definition, a prototype
   * Polygon cannot be defined of arbitrary size becaue Points are defined on
   * an integer grid.   Thus, a double scale factor is also provided that is 
   * used to scale the Polygon via the affine transform into a Shape of the desired size.
   * @param at The AffineTransform to use.
   * @param scaleFactor The ratio of the desired unit size to the defined size of the prototype Polygon.
   * @param pts Vararg parameters that are the Points that define the Polygon around the origin as its center. 
   */
  public PolygonFactory(AffineTransform at, double scaleFactor, Point2DDouble... pts) {
    this.at = at;
    this.scaleFactor = scaleFactor;
    for(Point2DDouble pt : pts) {
      poly.addPoint((int)Math.round(pt.x), (int) Math.round(pt.y));
    }
  }
  
  /**
   * Instantiates a Shape object that is the prototype Polygon translated
   * by the given (x, y) Point2DDouble and scaled by given (xScale, yScale) factor 
   * times the internal scaleFactor.
   * Note that this method does *not* return a Polygon object, but rather 
   * a abstract Shape object.  The return value *cannot* be cast to Polygon.
   * @param x x-coordinate of the center of the resulant Polygon
   * @param y y-coordinate of the center of the resulant Polygon
   * @param xScale The x-dimension of the polygon, usually the x-radius.
   * @param yScale The y-dimension of the polygon, usually the y-radius.
   * @return A Shape object that is the scaled prototype Polygon.
  */
  public Shape makeShape(double x, double y, double xScale, double yScale) {
    at.setToTranslation(x, y);
    at.scale(xScale*scaleFactor, yScale*scaleFactor);
    return at.createTransformedShape(poly);
  }
}