package ballwar.model.strategy;

import ballwar.model.*;
import util.*;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
 * Factory for a strategy that will set the *initial* color of a ball.
 * The currently selected color is copied into the strategy at the time it
 * is constructed.
 * Previously instantiated strategies are not affected by the current 
 * operation of the factory.
 */
public class ColorStrategyFac implements IStrategyFac { 
  private String _title = "ColorStrategy";
  
  private Color _color = Randomizer.Singleton.randomColor();
  private JColorChooser _colorChooser = new JColorChooser(_color);
  
  /**
   * Makes an IUpdateStrategy that initializes a ball to the color 
   * selected at the time this method was called.
   * @return A strategy with the current color saved internally.
   */
  public IUpdateStrategy make() {
    return new ANoOpStrategy(){
      private Color _localColor = _color; // Save the current color
      public void init(Ball host){
        /* 
         Don't want to use factory's _color here because the
         factory may have changed between the time the strategy 
         was made and the time the strategy is used it initialize 
         the host ball 
         */
        host.setColor(_localColor);
      }
    };
  }
  
  /**
   * The configuration panel for this factory.  Contains the 
   * JColorChooser component.
   */
  private JPanel _configPnl = new JPanel() {
    {
      add(_colorChooser);
    }
    public String toString() {
      return _title + " configuration";
    }
  };
  
  /**
   * The ActionListener for the Accept button of the configuration dialog 
   * form.   Saves the selected color value.
   */
  private ActionListener _acceptListener = new ActionListener() {
    public void actionPerformed(ActionEvent e) {
      // Save the color when the config is accepted
      _color = _colorChooser.getColor();
    }
  };
  
  /**
   * The ActionListener for the Cancel button of the configuration dialog 
   * form.   Resets the color chooser to the saved color value.
   */
  private ActionListener _cancelListener = new ActionListener() {
    public void actionPerformed(ActionEvent e) {
      // Reset the color chooser to the current color
      _colorChooser.setColor(_color);
    }
  };
  
  /**
   * Creates a configuration panel that presents a JColorChooser grid
   * to select a color.
   * The selected color is only saved when when the accept button is clicked.
   * @param acceptBtn  The button to click when the selected color is to be accepted.
   * @param cancelBtn The button to click when the selected color is to be discarded.
   * @return A configuration panel with the JColorChooser on it.
   */
  public JPanel getConfigPanel(AbstractButton acceptBtn, AbstractButton cancelBtn){
    acceptBtn.addActionListener(_acceptListener);
    cancelBtn.addActionListener(_cancelListener);
    return _configPnl;
  }
  
  public String toString() {
    return _title;
  }
}