package ballwar.model.strategy;

import ballwar.model.*;
import util.*;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
 * Factory to create strategies that make a ball move in a circle.
 * The selected turn angle will affect *ALL* strategies created by this
 * factory.
 * Different instances of this factory will not affect each other's strategies.
 */
public class CurveStrategyFac implements IStrategyFac { 
  private String _title = "CurveStrategy";
  
  /**
   * The incremental amount to rotate the context's velocity by each time its updateState is called (in radians).
   */
  private double _angle = Randomizer.Singleton.randomDouble(-Math.PI/4.0, Math.PI/4.0);
  
  /**
   * Make an IUpdateStrategy that will rotate the host ball's velocity 
   * at every update by the currently saved angle value.
   * @return A strategy tied to this factory's turn angle.
   */
  public IUpdateStrategy make() {
    return new ANoOpStrategy(){
      
      /**
       * Rotates the context's velocity by the specified angle.
       * @param context The strategy's context
       */
      public void updateState(Ball context) {
        double deltaTime = context.getEnv().getDeltaTime();
        // calculate current rotation parameters
        final double cosA = Math.cos(_angle*deltaTime);
        final double sinA = Math.sin(_angle*deltaTime);
        context.addUpdateCmd(new IBallCmd() {
          public void apply(Ball host) {
            // Need to get current velocity, in case it was
            // altered by something else
            Point2DDouble v = host.getVelocity();
            double x =cosA*v.getX()+ sinA*v.getY();
            double y = cosA*v.getY()-sinA*v.getX();
            v.setLocation(x, y);
          }
        });
      }
    };
  }
  
  /**
   * The text field to input the desired angle value
   */
  private JTextField _angleTF = new JTextField(10);
  
  /**
   * The configuration panel for the factory.  Contains
   * a label and a textfield on a panel.
   */
  private JPanel _configPnl = new JPanel() {
    {
      JPanel anglePnl = new JPanel();
      anglePnl.add(new JLabel("Turn angle (radians):"));
      _angleTF.setText(""+_angle);
      anglePnl.add(_angleTF);
      add(anglePnl);
    }
    public String toString() {
      return _title + " configuration";
    }
  };
  
  /**
   * The ActionListener for the Accept button of the configuration dialog 
   * form.   Parses the text in the textfield and saves it in the
   * _angle value.
   */
  private ActionListener _acceptListener = new ActionListener() {
    public void actionPerformed(ActionEvent e) {
      // save the value from the text field
      _angle = Double.parseDouble(_angleTF.getText());
    }
  };
  
  /**
   * The ActionListener for the Cancel button of the configuration dialog
   * form.  Resets the textfield back to the save angle value.
   */
  private ActionListener _cancelListener = new ActionListener() {
    public void actionPerformed(ActionEvent e) {
      _angleTF.setText(""+_angle); // reset the text field
    }
  };
  
  /**
   * Returns the configution panel for this factory, which has a color chooser
   * component on it for selecting a color to use when creating strategies.
   * @param acceptBtn The button to click when accepting a color choice.
   * @param cancelBtn The button to click when discarding a color choice.
   * @return The configuration panel.
   */
  public JPanel getConfigPanel(AbstractButton acceptBtn, AbstractButton cancelBtn){
    acceptBtn.addActionListener(_acceptListener);
    cancelBtn.addActionListener(_cancelListener);
    return _configPnl;
  }
  
  public String toString() {
    return _title;
  }
}