package ballwar.model.strategy;
import java.awt.*;
import ballwar.model.*;


/**
 * An ANoOpStrategy-based strategy that is the composition of two IUpdateStrategies.   
 * All behaviors are delegated sequentially to each of the composees. 
 */
public class MultiStrategy extends ANoOpStrategy {

  /**
	 * First IUpdateStrategy composee
	 */
	private IUpdateStrategy _s1;
  /**
	 * Second IUpdateStrategy composee
	 */
	private IUpdateStrategy _s2;
	/**
	 * Constructor for the class.
	 * @param s1 The first IUpdateStrategy composee
	 * @param s2 The second IUpdateStrategy composee
	 */
	public MultiStrategy(IUpdateStrategy s1, IUpdateStrategy s2) {
		_s1 = s1;
		_s2 = s2;
	}


   /**
	 * Updates the state of the context with the first composee and then the second composee.
	 * @param context This strategy's context.
	 */
	public void updateState(Ball context) {
    _s1.updateState(context);
    _s2.updateState(context);
  }


  
  /**
	 * Initializes the context with the first then the second composee.
	 * @param context This strategy's context.
	 */
	public void init(Ball context) {
    _s1.init(context);
    _s2.init(context);
  }

  
  /**
	 * Delegates first to the first composee and then to the second composee.
	 * @param context This strategy's context.
	 * @param target The ball the context collided with.
	 */
	public void updateCollision(Ball context, Ball target) {
    _s1.updateCollision(context, target);
    _s2.updateCollision(context, target);
  }
  /**
	 * Delegates the kill request to the first then the second composee.   Note that the first 
	 * composee may call the context's doKill() method, removing it from the system.   
	 * The context is not yet garbage collected, so the second composee can execute 
	 * normally, though perhaps without effect. 
	 * @param context This strategy's context.
	 */
	public void kill(Ball context) {
    _s1.kill(context);
    _s2.kill(context);
  }

}
