package ballwar.model.strategy;
import ballwar.model.*;
import java.awt.*;
import java.awt.geom.*;
import util.*;

/**
 * Creates a wall of finite length where balls bounce of only one side.
 * Use multiple one sided walls to create barriers of arbitrary shapes with 
 * interior and exterior bouncing surfaces.
 */
public class OneSideWallStrategy extends ANoOpStrategy {
  
  private Ball host;
  
  /**
   * One end of the wall
   */
  private Point2DDouble ptA;
  
  /**
   * Vector to the other end of the wall.
   * A vector V is approaching the wall from the outside
   * if V x vecS > 0.
   */
  private Point2DDouble vecS;

  
  
  private IBallCmd bounceCmd = new IBallCmd() {
    public void apply(Ball otherBall) {
      if(otherBall!=host) {
        
        Point2DDouble ptL = otherBall.getLocation();
        Point2DDouble vecV = otherBall.getVelocity();

        
        // vector from ptL to ptA
        Point2DDouble vecLA = ptL.vecTo(ptA);

        
        // Calculate the perpendicular distance to the other ball
        
        double dist = -vecS.crossProd(vecLA)/vecS.getLength();
        
        if(Math.abs(dist) > otherBall.getRadius()) return; // too far away
       // System.out.println("dist = "+dist + " radius = "+otherBall.getRadius());
        
        
        
        double crossVS = vecV.crossProd(vecS);
        
        if(crossVS <= 0.0) return;  // wall and velocity are parallel or wrong direction

        // first get unit vector normal to vecS, pointing outward
        final Point2DDouble normal = new Point2DDouble(-vecS.y/vecS.getLength(), vecS.x/vecS.getLength());

        // get copy of normal b/c unmutated normal is needed later.
        final Point2DDouble tempNormal = new Point2DDouble(normal);
        
        // see which side of the line the ball is on
        // and find location of nearest point on circular ball to wall
        if(normal.innerProd(vecLA)>0) {
          tempNormal.multBy(-otherBall.getRadius());
        }
        else {
          tempNormal.multBy(otherBall.getRadius());
        }
        // Calculate trajectory of nearest point of ball
        vecLA.translate(tempNormal);
        
        double crossVLA = vecV.crossProd(vecLA);
        
        double t1 = -crossVLA/crossVS;
        if((t1 > 1.0) || (t1 < 0.0)) return; // intersection is beyond wall
        //System.out.println("t1 = "+t1);
        
        
        // actually bounce the other ball
        normal.multBy(-2.0*normal.innerProd(vecV));
        
        tempNormal.multBy(-1.1*(1.-Math.abs(dist)/otherBall.getRadius()));
        
        // set up command for later processing
        otherBall.addUpdateCmd(new IBallCmd() {
          public void apply(Ball host) {
            host.getVelocity().translate(normal); // change velocity
            host.getLocation().translate(tempNormal); // push ball outside of wall
          }
        });
        
      }
    }
  };
  
  private Color wallColor = Color.BLACK;
  
  private IPaintStrategy paintStrategy = new IPaintStrategy() {
      public void paint(Graphics g, Ball host) {
        g.setColor(wallColor);
        g.drawLine((int) Math.round(ptA.x),
                   (int) Math.round(ptA.y),
                   (int) Math.round(ptA.x+vecS.x),
                   (int) Math.round(ptA.y+vecS.y));
      }
  
      public void paintXfrm(Graphics g, Ball host, AffineTransform at) {}
  
      public void init(Ball host) {}
    };
  
  public OneSideWallStrategy(Point2DDouble ptA, Point2DDouble vecS, Color wallColor){
    this.ptA = ptA;
    this.vecS = vecS;
    this.wallColor = wallColor;
  }
  
  public OneSideWallStrategy() {
    this(new Point2DDouble(500*(Math.random()), 500*(Math.random())),
         new Point2DDouble(500*(Math.random()-.5), 500*(Math.random()-.5)),
         Color.RED);
  }
  
  public void updateState(Ball context) {
    context.getEnv().getDispatcher().notifyAll(bounceCmd);
  }

  public void init(Ball context) {
    host = context;
    host.setLocation(ptA);
    host.setMoveStrategy(new IBallMoveStrategy() {
      public void move(Ball host) {}
  
      private Point2DDouble vel = new Point2DDouble(0,0);
      public Point2DDouble getVelocity() {
        return vel;
      }
      public void setVelocity(Point2DDouble v) {}
    });
    
    host.setPaintStrategy(paintStrategy);
  }
  
  public IPaintStrategy getPaintStrategy() {
    return paintStrategy;
  }
  
}
