
/**
 * Title:        Ball World<p>
 * Description:  <p>
 * Copyright:    Copyright (c) DXN, SBW<p>
 * Company:      Rice University<p>
 * @author DXN
 * @version 1.0
 */
package ballwar.view;

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import util.*;


/**
 * A subclass of JFrame containing the various GUI components specified in the program behavior specification.
 * Allows the balls to be specified, constructed, drawn and controlled.   
 * Allows players to be specified and the game to be controlled.
 * Provides a Container area where the drawing object is to be drawn.
 */
public class BallGUI extends JFrame {
  
  
  /**
   * The panel upon which all the game controls are located.
   */
  private JPanel _controlPnl = new JPanel();
  
  /**
   * The panel upon which the balls are painted.
   */
  private JPanel _canvasPnl = new JPanel()  {
    public void paint(Graphics g) {
      super.paint(g);
      _bCtrlA.paint(g);
    }
  };
  /**
   * The button to clear all the balls.
   */
  private JButton _clearAllBtn = new JButton("ClearAll");
  /**
   * The textfield where the user can type in the classname of a new strategy.
   */
  private JTextField _inputTF = new JTextField("Straight");
  /**
   * Button to add the strategy entered in inputTF to both drop lists.
   */
  private JButton _addBtn = new JButton("Add strategy to lists");
  /**
   * Button to add the strategy factory entered in inputTF to both drop lists.
   */
  private JButton _addFacBtn = new JButton("Add factory to lists");

  /**
   * The sub-panel that the input textfield is located.
   */
  private JPanel _inputPnl = new JPanel();
  /**
   * The button that makes a new ball with a SwitcherStrategy.
   */
  private JButton _makeSwitcherBtn = new JButton("Make Switcher");
  /**
   * The button that switches the decoree in the SwitcherStrategy to the currently selected strategey.
   */
  private JButton _switchBtn = new JButton("Switch!");
  /**
   * The sub-panel that holds the SwitcherStrategy controls.
   */
  private JPanel _switcherPnl = new JPanel();
  /**
   * The sub-panel that holds the drop-lists of strategies and their control buttons.
   */
  private JPanel _cbPnl = new JPanel();
  /**
   * A drop-list with all the strategies and combinations of strategies.   Holds the factories that are used by the model to instantiate 
   * balls with that strategy.   The factory's toString() method is called to generate the text shown on 
   * the drop list.   This drop list is the list used when the user selects a strategy, except when creating a combination.
   */
  private JComboBox _list1CBox = new JComboBox();
  /**
   * Same content as _list1CBox because two lists are needed to create combinations 
   * of strategies.
   */
  private JComboBox _list2CBox = new JComboBox();
  /**
   * Button used to combine a selection from _list1CBox and a selection from _list2BCBox into a combined strategy.
   */
  private JButton _combineBtn = new JButton("Combine!");
  /**
   * Button to make a ball with the selected strategy in _list1CBox.
   */
  private JButton _makeSelectedBtn = new JButton("Make Selected Ball");

  /**
   * Button to configure the selected strategy in _list1CBox.
   */
  private JButton _configSelectedBtn = new JButton("Configure Selected Ball");
  
  /**
   * Label at the bottom of the screen used to display the status of the game and the player's scores.
   * 
   */
  private JLabel _statusLbl = new JLabel("Status: ");
  /**
   * Display command used to generate the latest display string.
   */
  private ILambda _displayCmd = ILambda.NullObject;
  /**
   * Button to make a new player with the name typed in by the user.
   */
  private JButton _makePlayerBtn = new JButton("Make Player");
  /**
   * Drop list that shows the names of all the players made so far.   Holds players and uses their 
   * toString() method to display their names.
   */
  private JComboBox _playersCBox = new JComboBox();
  
  /**
   * The textfield where the user can type in the classname of a new painting strategy.
   */
  private JTextField _paintTF = new JTextField("Ball");
  
  /**
   * The sub-panel that the paint selection controls are located.
   */
  private JPanel _paintSelPnl = new JPanel();
  
  /**
   * Button to add the paint strategy entered in _paintTF to the drop list.
   */
  private JButton _addPaintBtn = new JButton("Add Paint Strategy");
  
  /**
   * Drop list with available paint strategies.
   */
  private JComboBox _paintCBox = new JComboBox();
  
  /**
   * The IBallCtrlAdapter used to control the balls.
   */
  private IBallCtrlAdapter _bCtrlA;
  /**
   * The IGameCtrlAdapter used to control the game.
   */
  private IGameCtrlAdapter _gCtrlA;
  
  /**
   * Constructor for the class.   Attaches the given adapters to their respective behaviors.  
   * Notice how the adapter functionalities are grouped such that related behaviors are 
   * on the same adapter.   The view (this class) has no knowledge of the actual organization 
   * of the model, for instance, that it is even represented by a single class.  
   * 
   * @param wl The WindowListener called when this window closes.
   * @param bCreateA The IBallCreateAdapter used to create the balls.
   * @param bCtrlA The IBallCtrlAdapter used to control the balls.
   * @param gCtrlA The IGameCtrlAdapter used to control the game.
   */
  public BallGUI( WindowListener wl, final IBallCreateAdapter bCreateA, IBallCtrlAdapter bCtrlA, IGameCtrlAdapter gCtrlA) {
    addWindowListener(wl);
    _bCtrlA = bCtrlA;  
    _gCtrlA = gCtrlA;
    
    _clearAllBtn.addActionListener(new ActionListener(){
      public void actionPerformed(ActionEvent e) {
        _bCtrlA.clearBalls();
      }
    });
    
    _makeSwitcherBtn.addActionListener(new ActionListener(){
      public void actionPerformed(ActionEvent e) {
        bCreateA.makeSwitcher(_paintCBox.getSelectedItem());
      }
    });
    
    _switchBtn.addActionListener(new ActionListener(){
      public void actionPerformed(ActionEvent e) {
        _bCtrlA.switchStrategy(_list1CBox.getSelectedItem());
      }
    });
    
    _addBtn.addActionListener(new ActionListener(){
      public void actionPerformed(ActionEvent e) {
        Object o = bCreateA.addStrategy(_inputTF.getText());
        if(null==o) return;
        _list1CBox.insertItemAt(o,0);
        _list2CBox.insertItemAt(o,0);
      }
    });

    _addFacBtn.addActionListener(new ActionListener(){
      public void actionPerformed(ActionEvent e) {
        Object o = bCreateA.addStrategyFac(_inputTF.getText());
        if(null==o) return;
        _list1CBox.insertItemAt(o,0);
        _list2CBox.insertItemAt(o,0);
      }
    });
    
    _combineBtn.addActionListener(new ActionListener(){
      public void actionPerformed(ActionEvent e) {
        Object o = bCreateA.combineStrategies(_list1CBox.getSelectedItem(),_list2CBox.getSelectedItem());
        if(null==o) return;
        _list1CBox.insertItemAt(o,0);
        _list2CBox.insertItemAt(o,0);
      }
    });
    
    _configSelectedBtn.addActionListener(new ActionListener(){
      public void actionPerformed(ActionEvent e) {
        AConfigDlg configDlg = new AConfigDlg(BallGUI.this,bCreateA, _list1CBox.getSelectedItem()){};
        configDlg.setVisible(true);
      }
    });
    
    _makeSelectedBtn.addActionListener(new ActionListener(){
      public void actionPerformed(ActionEvent e) {
        bCreateA.makeBall(_list1CBox.getSelectedItem(), _paintCBox.getSelectedItem());
      }
    });
    
    _addPaintBtn.addActionListener(new ActionListener(){
      public void actionPerformed(ActionEvent e) {
        Object o =bCreateA.addPaintStrategy(_paintTF.getText());
        if(null==o) return;
        _paintCBox.insertItemAt(o,0);
      }
    });    
    try {
      init();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
    
    // Convenience code to preselect the update and paint strategies
    // that are already listed in the textboxes.
    _addBtn.doClick();
    _list1CBox.setSelectedIndex(0);
    _addPaintBtn.doClick();
    _paintCBox.setSelectedIndex(0);
  }
  
  /**
   * Initializes and lays out the GUI components.
   */
  private void init() throws Exception {
    this.setTitle("Ball World");
    this.getContentPane().setLayout(new BorderLayout());
    this.getContentPane().add(_controlPnl, BorderLayout.NORTH);
    //_canvasPnl.setBackground(Color.BLACK);
    this.getContentPane().add(_canvasPnl, BorderLayout.CENTER);
    this.getContentPane().add(_statusLbl, BorderLayout.SOUTH);
    _inputPnl.setLayout(new GridLayout(0,1));
    _controlPnl.add(_inputPnl);
    _inputPnl.add(_inputTF);
    _inputTF.setToolTipText("Name of strategy,XXX, from ballwar.strategy.XXXStrategy.");
    _inputPnl.add(_addBtn);
    _addBtn.setToolTipText("Add strategy to both droplists.  The rest of the of classname is added automatically.");
    _inputPnl.add(_addFacBtn);
    _addFacBtn.setToolTipText("Add strategy factory to both droplists.  The rest of the of classname is added automatically.");
    _cbPnl.setLayout(new GridLayout(0,1));
    _controlPnl.add(_cbPnl);
    _cbPnl.add(_configSelectedBtn);
    _cbPnl.add(_makeSelectedBtn);
    _makeSelectedBtn.setToolTipText("Instantiate a Ball with the strategy selected in the upper droplist.");
    _cbPnl.add(_list1CBox);
    _cbPnl.add(_list2CBox);
    _cbPnl.add(_combineBtn);
    _combineBtn.setToolTipText("Combine the selected strategies from both droplists into a single strategy that is added to both droplists.");
    _switcherPnl.setLayout(new GridLayout(0,1));
    _controlPnl.add(_switcherPnl);
    _switcherPnl.add(_makeSwitcherBtn);
    _makeSwitcherBtn.setToolTipText("Instantiate a ball that can switch strategies.");
    _switcherPnl.add(_switchBtn);
    _switchBtn.setToolTipText("Switch the strategy on all switcher balls to the currently selected strategy in the upper droplist.");
    _controlPnl.add(_clearAllBtn); 
    _clearAllBtn.setToolTipText("Clear all the balls from the screen.");
    
    _paintSelPnl.setLayout(new GridLayout(0,1));
    _paintSelPnl.add(_paintTF);
    _paintSelPnl.add(_addPaintBtn);
    _paintSelPnl.add(_paintCBox);
    _controlPnl.add(_paintSelPnl);
    
    
    Box playerBox = Box.createVerticalBox();
    _controlPnl.add(playerBox);
    final JTextField playerNameTF = new JTextField();
    _makePlayerBtn.setAlignmentX((float)0.5);
    playerBox.add(playerNameTF);
    playerBox.add(_makePlayerBtn);
    playerBox.add(_playersCBox);
    _makePlayerBtn.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if(0<playerNameTF.getText().length()) {   // player must have name
          _playersCBox.addItem(_gCtrlA.makePlayer(playerNameTF.getText()));
        }
        validate();
      }
    });
    _canvasPnl.addMouseListener(new MouseAdapter() {
      public void mouseClicked(MouseEvent e) {
        _canvasPnl.requestFocus();
      }
    });
    pack();
    setSize (new Dimension(getWidth(), 800));
    setLocationRelativeTo(null);
  }
  
  /**
   * Automaticaly adjusts the width of the GUI to its minumum size such that all the controls can 
   * be seen.   The height is unaffected.   Useful when new items are added to the drop lists, changing the 
   * required width of the GUI.
   */
  public void autoWidth() {
    int height = getHeight();
    validate();
    pack();
    setSize (new Dimension(getWidth(), height));
  }
  
  
  /**
   * Accessor method for the panel upon which the balls are to be drawn
   */
  public Container getContainer()
  {
    return _canvasPnl;
  }
  
 
  /**
   * Updates the view.   This will update the display string shown on the status label.   
   * This will also, in turn, ask the model to update.   This is required because the model 
   * has the timer which tells the system when to update, but the model cannot update
   * until the view is ready to repaint.
   */
  public void update() {
    _canvasPnl.repaint();
    _statusLbl.setText(_bCtrlA.getDisplayStr("Status: "));
  }
  
  /**
   * Adds the given key such that whenever that key is pressed the given command (AbstractAction) is performed.  
   * If the given key was already added, its behavior is replaced by the new behavior.  
   * @param keyName The name of the virtual key to add, minus the leading "VK_".   See the Java documentation under "KeyEvent". 
   * @param cmd The behavior to associate with this key.   The command's actionPerformed() method will be called whenever the given key is pressed.
   */
  public void addKeyCmd(final String keyName, final ILambda cmd) {
    _canvasPnl.getInputMap().put(KeyStroke.getKeyStroke(keyName),keyName+cmd);
    _canvasPnl.getActionMap().put(keyName+cmd, new AbstractAction() {
      public void actionPerformed(ActionEvent e) {
        cmd.apply(keyName);
      }
    });    
  }
  
  /**
   * Returns the selected player (held in a drop list) .   The GUI has no knowledge of the type or usage 
   * of the player object.
   * 
   * @return The selected player.
   */
  public Object getSelectedPlayer() {
    return _playersCBox.getSelectedItem();
  }
  
  /**
   * Adds the given Component to the control panel.   Used to add such things as 
   * new OptionPanels to the control panel without requiring that the GUI know what 
   * their functionality.
   * @param c The component to add.
   */
  public void addComponent(Component c) {
    _controlPnl.add(c);
    autoWidth();
  }
  
}